dir.create("Figures", showWarnings = FALSE)
dir.create("Output", showWarnings = FALSE)

## figure 1 ##
library("BSGW")
library("Hmisc")
library("CFC")
library("survival")
data("bmt", package = "CFC")

predict.S.core <- function(t, xbeta, xbetas, alpha.min, alpha.max, ordweib) {
    if (t == 0.0) return(1.0)
    if (ordweib) {
        alpha <- exp(xbetas)
    } else {
        alpha <- alpha.min + (alpha.max-alpha.min)/(1+exp(-xbetas))
    }
    H <- t^alpha * exp(xbeta)
    S <- exp(-H)
    return(S)
}

operator <- mean
  
iter <- 5000
ordweib <- TRUE
  
lung <- lung[complete.cases(lung), ]
  
seed.no <- 0
set.seed(seed.no)
est <- bsgw(Surv(futime, fustat) ~ as.factor(ecog.ps) + as.factor(rx), ovarian, ordweib = ordweib,
            control = bsgw.control(iter = iter, nskip = 100, scalex = FALSE), print.level = 0)

X <- est$X
Xs <- est$Xs
beta.smp <- est$smp$beta
betas.smp <- est$smp$betas
alpha.min <- est$control$alpha.min
alpha.max <- est$control$alpha.max

beta.avg <- apply(beta.smp[(iter/2+1):iter, ], 2, operator)
betas.avg <- apply(betas.smp[(iter/2+1):iter, , drop = F], 2, operator)

t.compare <- round(0.5 * est$tmax)

xbeta.avg <- X %*% beta.avg
xbetas.avg <- Xs %*% betas.avg
S.wrong <- predict.S.core(t.compare, xbeta.avg, xbetas.avg, alpha.min, alpha.max, ordweib)

xbeta.smp <- X %*% t(beta.smp)
xbetas.smp <- Xs %*% t(betas.smp)
S.smp <- sapply(1:iter, function(i) {
    predict.S.core(t.compare, xbeta.smp[, i], xbetas.smp[, i], alpha.min, alpha.max, ordweib)
})
S.right <- apply(S.smp[, (iter/2+1):iter], 1, operator)
S.right.sd <- apply(S.smp[, (iter/2+1):iter], 1, sd)
S.right.se <- S.right.sd / sqrt(iter/2)
S.right.plus <- S.right + 2 * S.right.se
S.right.minus <- S.right - 2 * S.right.se

my.range.x <- range(S.wrong, S.right)
my.range.y <- my.range.x

pdf(file = file.path("Figures", "fig1.pdf"), width = 10)
old.par <- par(mfrow = c(1,2))

errbar(x = S.wrong, y = S.right, yplus = S.right.plus, yminus = S.right.minus,
       ylim = my.range.y, xlim = my.range.x,
       ylab = "Bayesian survival probability", xlab = "non-Bayesian survival probability")
abline(a = 0, b = 1)
  
days.in.year <- 365.25
fitKM <- survfit(Surv(stop, event == "pcm") ~ 1, data = mgus1,
                 subset = (start == 0))
fitCI <- survfit(Surv(stop, status*as.numeric(event), type = "mstate") ~ 1,
                 data = mgus1, subset = (start == 0))

## NOTE: in older versions of survival, we must use fitCI$prev rather than fitCI$pstate
ytmp <- if (is.null(fitCI$pstate)) fitCI$prev[, 1] else fitCI$pstate[, 1]
plot(fitCI$time / days.in.year, ytmp, type = "l",
   ylim = c(0.0, 0.4), xlim = c(0.0, 7300) / days.in.year,
   xlab = "Years post diagnosis of MGUS", ylab = "Incidence Probability",
   lwd = 2)
lines(fitKM$time / days.in.year, 1 - fitKM$surv, lty = 2, lwd = 2)
legend("topleft", legend = c("Competing-Risk", "Kaplan-Meier"), lty = 1:2, lwd = rep(2, 1))
  
par(old.par)
dev.off()

## example 1 ##
rm(list = ls())

library("CFC")
data("bmt", package = "CFC")
seed.no <- 0
set.seed(seed.no)

rel.tol <- 1e-3
idx.train <- sample(1:nrow(bmt), size = 0.7 * nrow(bmt))
idx.pred <- setdiff(1:nrow(bmt), idx.train)
nobs.train <- length(idx.train)
nobs.pred <- length(idx.pred)

out.weib <- cfc.survreg(Surv(time, cause) ~ platelet + age + tcell, 
                        bmt[idx.train, ], bmt[idx.pred, ], rel.tol = rel.tol)
summ <- summary(out.weib, obs.idx = which(bmt$age[idx.pred] > 0))
pdf(file = file.path("Figures", "fig2.pdf"), width = 10)
plot(summ, which = 1)
dev.off()

pdf(file = file.path("Figures", "fig3.pdf"), width = 12)
old.par <- par(mfrow = c(1, 2)); plot(summ, which = 2); par(old.par)
dev.off()

## examples 2 & 3
rm(list = ls())
  
## example 2
library("survival")
library("CFC")
data("bmt", package = "CFC")
out.prep <- cfc.prepdata(Surv(time, cause) ~ platelet + age + tcell, bmt)
f1 <- out.prep$formula.list[[1]]
f2 <- out.prep$formula.list[[2]]
dat <- out.prep$dat
tmax <- out.prep$tmax

library("BSGW")
seed.no <- 0
set.seed(seed.no)
idx.train <- sample(1:nrow(bmt), size = 0.7 * nrow(bmt))
idx.pred <- setdiff(1:nrow(bmt), idx.train)
nobs.train <- length(idx.train)
nobs.pred <- length(idx.pred)
nsmp <- 10
reg1 <- bsgw(f1, dat[idx.train, ],
             control = bsgw.control(iter = nsmp),
             ordweib = TRUE, print.level = 0)
reg2 <- bsgw(f2, dat[idx.train, ], 
             control = bsgw.control(iter = nsmp),
             ordweib = TRUE, print.level = 0)
  
survfunc <- function(t, args, n) {
    nobs <- args$nobs; natt <- args$natt; nsmp <- args$nsmp
    alpha <- args$alpha; beta <- args$beta; X <- args$X
    idx.smp <- floor((n - 1) / nobs) + 1
    idx.obs <- n - (idx.smp - 1) * nobs
    return(exp(- t ^ alpha[idx.smp] * 
               exp(sum(X[idx.obs, ] * beta[idx.smp, ]))))
}
f.list <- list(survfunc, survfunc)

X.pred <- as.matrix(cbind(1, bmt[idx.pred, c("platelet", "age", "tcell")]))
arg.1 <- list(nobs = nobs.pred, natt = 4, nsmp = nsmp,
              X = X.pred, alpha = exp(reg1$smp$betas),
              beta = reg1$smp$beta)
arg.2 <- list(nobs = nobs.pred, natt = 4, nsmp = nsmp, 
              X = X.pred, alpha = exp(reg2$smp$betas), 
              beta = reg2$smp$beta)
arg.list <- list(arg.1, arg.2)

rel.tol <- 1e-4
tout <- seq(from = 0.0, to = tmax, length.out = 10)
t.R <- proc.time()[3]
out.cfc.R <- cfc(f.list, arg.list, nobs.pred * nsmp, tout,
                 rel.tol = rel.tol)
t.R <- proc.time()[3] - t.R
cat("t.R:", t.R, "sec\n")
saveRDS(t.R, file = file.path("Output", "t_R.rds"))

my.f.reduce <- function(x, nobs, nsmp) {
    return (colMeans(array(x, dim = c(nobs, nsmp))))
}
summ.R <- summary(out.cfc.R, f.reduce = my.f.reduce, nobs = nobs.pred, nsmp = nsmp)
pdf(file = file.path("Figures", "fig4.pdf"), width = 12, height = 12)
oldpar <- par(mfrow = c(2, 2))
plot(summ.R)
par(oldpar)
dev.off()

ncores <- 2
tout <- seq(from = 0.0, to = tmax, length.out = 10)
t.R.par <- proc.time()[3]
out.cfc.R.par <- cfc(f.list, arg.list, nobs.pred * nsmp, tout,
                     rel.tol = rel.tol, ncores = ncores)
t.R.par <- proc.time()[3] - t.R.par
cat("t.R.par:", t.R.par, "sec\n")
saveRDS(t.R.par, file = file.path("Output", "t_R_par.rds"))
saveRDS(out.cfc.R, file = file.path("Output", "out_cfc_R.rds"))

## example 3
tout <- seq(from = 0.0, to = tmax, length.out = 10)
library("Rcpp")
Rcpp::sourceCpp("weib.cpp")
f.list.Cpp.1 <- list(weib_getPtr_func(), weib_getPtr_init(),
                     weib_getPtr_free())
f.list.Cpp <- list(f.list.Cpp.1, f.list.Cpp.1)
t.Cpp <- proc.time()[3]
out.cfc.Cpp <- cfc(f.list.Cpp, arg.list, nobs.pred * nsmp, tout,
                   rel.tol = rel.tol)
t.Cpp <- proc.time()[3] - t.Cpp
cat("t.Cpp:", t.Cpp, "sec\n")
saveRDS(t.Cpp, file = file.path("Output", "t_Cpp.rds"))
saveRDS(out.cfc.Cpp, file = file.path("Output", "out_cfc_Cpp.rds"))

print(all.equal(out.cfc.R, out.cfc.Cpp))

nsmp <- 1000
reg1 <- bsgw(f1, dat[idx.train, ],
             control = bsgw.control(iter = nsmp),
             ordweib = T, print.level = 0)
reg2 <- bsgw(f2, dat[idx.train, ], 
             control = bsgw.control(iter = nsmp),
             ordweib = T, print.level = 0)
arg.1 <- list(nobs = nobs.pred, natt = 4, nsmp = nsmp,
              X = X.pred, alpha = exp(reg1$smp$betas),
              beta = reg1$smp$beta)
arg.2 <- list(nobs = nobs.pred, natt = 4, nsmp = nsmp, 
              X = X.pred, alpha = exp(reg2$smp$betas), 
              beta = reg2$smp$beta)
arg.list <- list(arg.1, arg.2)
t.Cpp.1000 <- proc.time()[3]
out.cfc.Cpp.1000 <- cfc(f.list.Cpp, arg.list, nobs.pred * nsmp, tout,
                        rel.tol = rel.tol)
t.Cpp.1000 <- proc.time()[3] - t.Cpp.1000
cat("t.Cpp - 1000 samples:", t.Cpp.1000, "sec\n")
saveRDS(t.Cpp.1000, file = file.path("Output", "t_Cpp_1000.rds"))

ncores <- 2
t.Cpp.1000.par <- proc.time()[3]
out.cfc.Cpp.1000.par <- cfc(f.list.Cpp, arg.list, nobs.pred * nsmp, tout,
                            rel.tol = rel.tol, ncores = ncores)
t.Cpp.1000.par <- proc.time()[3] - t.Cpp.1000.par
cat("t.Cpp.par - 1000 samples:", t.Cpp.1000.par, "sec\n")
saveRDS(t.Cpp.1000.par, file = file.path("Output", "t_Cpp_1000_par.rds"))

cat("parallelization speedup - C++ - 1000 samples:", t.Cpp.1000 / t.Cpp.1000.par, "\n")

## example 4
library("CFC")
library("survival")
library("randomForestSRC")
data("bmt", package = "CFC")

seed.no <- 0
set.seed(seed.no)

prep <- cfc.prepdata(Surv(time, cause) ~ platelet + age + tcell, bmt)
f1 <- prep$formula.list[[1]]
f2 <- prep$formula.list[[2]]
dat <- prep$dat
tmax <- prep$tmax

reg1 <- survreg(f1, dat, x = TRUE)

reg2 <- rfsrc(f2, dat)

rfsrc.survfunc <- function(t, args, n) {
    which.zero <- which(t < .Machine$double.eps)
    ret <- approx(args$time.interest, args$survival[n, ], t, rule = 2)$y
    ret[which.zero] <- 1.0
    return (ret)
}
f.list <- list(cfc.survreg.survprob, rfsrc.survfunc)
arg.list <- list(reg1, reg2)
tout <- seq(0.0, tmax, length.out = 10)
cfc.out <- cfc(f.list, arg.list, nrow(bmt), tout, rel.tol = 1e-4)

pdf(file = file.path("Figures", "fig5.pdf"), width = 10)
oldpar <- par(mfrow = c(2, 2))
plot(summary(cfc.out, f.reduce = mean))
par(oldpar)
dev.off()





